#!/usr/bin/python3
# -*- coding:euc-jp -*-

import argparse
import configparser
import sys
import re
import os
import collections
import math
sys.path.append('MY_MODULES')
import READ_INPUT

#==========================================================================
# make tex source file to svg file
#   %test strings in tex source is replaced to 
#              first string of line in command file.
#   output svg file name should be write 
#              in secound string of line in command file.   
#==========================================================================
class tex2svg:
    def __init__(self, source_file, input_data):
        self.source_f  = source_file
        self.pattern   = '%test'
        self.tex_f     = "temp.tex"
        self.dvi_f     = self.tex_f.replace(r".tex", ".dvi")
        self.pdf_f     = self.tex_f.replace(r".tex", ".pdf")
        self.svg_f     = self.tex_f.replace(r".tex", ".svg")
        self.data = input_data
        self.read_source()

    def read_source(self):
        souf = open(self.source_f, "r", encoding="euc_jp")
        self.source = souf.readlines()
        souf.close()

    def write_source(self, i):
        tex_cmd = self.data['command'][i]['TeX']
        temp_tex = open(self.tex_f, "w", encoding="euc_jp")
        for line in self.source:
            if re.search(self.pattern, line):
                line = line.replace(self.pattern, tex_cmd)
                temp_tex.write(line)
                next
            else:
                temp_tex.write(line)
        temp_tex.close()

    def platex(self):
        os.system("platex {0:s}".format(self.tex_f))

    def make_svg(self, i):
        os.system("dvisvgm --no-fonts -c {0:s} -e {1:s}".\
                  format(self.data['html']['svg_size']['height'].replace('px', ''),\
                         self.dvi_f))
        os.system("mv {0:s} {1:s}".
                  format(self.svg_f,
                         os.path.join('../', self.data['command'][i]['output'])))
        os.system("chmod 644 {0:s}".
                  format(os.path.join('../', self.data['command'][i]['output'])))
        
    def delete_temp_file(self):
        os.system("rm temp.*")


#==========================================================================
# make table of HTML 
#==========================================================================
class make_HTML_table:
    def __init__(self, input_data, file="../table.html"):
        self.data = input_data
        self.table_file = file

    def make_html(self):
        def mk_table():
            table_id = self.data['html']['dir'].replace('/', ':')
            if table_id[-1] == ':':
                table_id = table_id[:-1]
            html = '<table id="{0:s}" class="latex_command_result" summary="{1:s}">\n'.\
                   format(table_id, self.data['html']['table_summary'])
            return html

        def mk_caption():
            line = '<caption>{0:s}</caption>\n'
            html = line.format(self.data['html']['table_caption'])
            return html

        def mk_col():   # <col>
            line_ary = []
            html = ""
            line = '<col style="width:{0:s}">\n'
            line00 = line.format(self.data['html']['col_width']['command'])
            line01 = line.format(self.data['html']['col_width']['output'])
            line02 = line.format(self.data['html']['col_width']['space'])
            line_ary = [line00, line01, line02]
            for i in range(0, self.data['html']['ncolumn']*3-1):
                index = i % 3
                html += line_ary[index]
            else:
                html += "\n"
            return html

        def mk_th(command, result):     # <tr><th>
            line = []
            html = '<tr>\n'
            line = '<th class="{0:s}">{1:s}</th>\n'
            line00 = line.format('command', command)
            line01 = line.format('result', result)
            line02 = line.format('separater', '')
            line = [line00, line01, line02]
            for i in range(0, self.data['html']['ncolumn']*3-1):
                index = i % 3
                html += line[index]
            else:
                html += '</tr>\n\n'
            return html

        def mk_td(j, svg_height, command, img_path):  #<tr><td>
            line = []
            html = ''
            line00 = '<td class="command">{0:s}</td>\n'.format(command)
            line01 = '<td class="result" style="text-align:{0:s}; padding-left:{1:s}">'.\
                       format(self.data['html']['align'], self.data['html']['padding'])
            line01 += '<a href="{0:s}" data-lightbox="{1:s}" data-title="{2:s}">'.\
                        format(img_path, self.data['html']['LB_grp'], command)
            line01 += '<img style="height:{0:s}" src="{1:s}"></a></td>\n'.\
                        format(self.data['command'][j]['height'], img_path)
            line02 = '<td class="separater"> </td>\n'
            if j == self.data['html']['ncolumn']-1:
                html = line00 + line01
            else:
                html = line00 + line01 + line02
            return html

        def mk_LightBox(img, group, title):
            line = '<a href="{0}" data-lightbox="{1}" data-title="{2}">'
            html = line.format(img, group, title)
            return html


        nrow = math.ceil(len(self.data['command'])/self.data['html']['ncolumn'])
        fd = open(self.table_file, 'w', encoding="euc_jp")

        fd.write(mk_table())
        fd.write(mk_caption())
        fd.write(mk_col())
        fd.write(mk_th("ޥ", ""))

        for i in range(0, nrow):
            fd.write("<tr>\n")
            for j in range(0, self.data['html']['ncolumn']):
                k = i*self.data['html']['ncolumn'] + j
                if len(self.data['command']) <= k : 
                    fd.write('</tr>\n\n')
                    break
                command  = self.data['command'][k]['TeX']
                img_path = os.path.join(self.data['html']['dir'],
                                        self.data['command'][k]['output'])
                svg_height = self.data['html']['svg_size']['height']
                fd.write(mk_td(k, svg_height, command, img_path))

            else:
                fd.write('</tr>\n\n')
        else:
            fd.write("</table>")        
        fd.close()
        os.system("chmod 644 {0:s}".format(self.table_file))
        

#------------------------------------------------
# like a main function
#   argument  : ./tex2svg  *.tex  commd_file
#------------------------------------------------
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='LaTeX Υޥɤȥȥץå(svg)ν')
    parser.add_argument('-t', help='LaTeX soure file')
    parser.add_argument('-i', help='input file')
    args = parser.parse_args()

    # --- ϥǡɤ߹ ---
    input_data = READ_INPUT.INPUT(args.i)
    input_data = input_data.get_input_datas()

    # --- svg κ ---
    svg = tex2svg(args.t, input_data)
    for i in range(0, len(input_data['command'])):
        svg.write_source(i)
        svg.platex()
        svg.make_svg(i)
        svg.delete_temp_file()

    HTML_table = make_HTML_table(input_data)
    HTML_table.make_html()

    del(svg)
    del(HTML_table)
